@@ -14,18 +14,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import fixtures
-
 import integration_tests
 
 
 class MavenPluginTestCase(integration_tests.TestCase):
 
-    def setUp(self):
-        super().setUp()
-        self.useFixture(
-            fixtures.EnvironmentVariable('SNAPCRAFT_SETUP_PROXIES', '1'))
-
     def test_build_maven_plugin(self):
         project_dir = 'simple-maven'
         self.run_snapcraft('build', project_dir)
