@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015-2016 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -16,7 +16,7 @@
 
 import subprocess
 
-from testtools.matchers import EndsWith
+from testtools.matchers import Contains
 
 import integration_tests
 
@@ -30,7 +30,19 @@ class StageTestCase(integration_tests.TestCase):
             self.run_snapcraft, 'stage', project_dir)
 
         self.assertEqual(1, exception.returncode)
-        expected = (
+        expected_conflicts = (
             "Parts 'p1' and 'p2' have the following file paths in common "
-            "which have\ndifferent contents: bin/test\n")
-        self.assertThat(exception.output, EndsWith(expected))
+            "which have different contents:\n    bin/test\n")
+        self.assertThat(exception.output, Contains(expected_conflicts))
+
+        expected_help = (
+            'Snapcraft offers some capabilities to solve this by use '
+            'of the following keywords:\n'
+            '    - `filesets`\n'
+            '    - `stage`\n'
+            '    - `snap`\n'
+            '    - `organize`\n\n'
+            'Learn more about these part keywords by running '
+            '`snapcraft help plugins`'
+        )
+        self.assertThat(exception.output, Contains(expected_help))
