@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from snapcraft.internal.states._strip_state import StripState  # noqa
+from snapcraft.internal.states._prime_state import PrimeState  # noqa
 from snapcraft.internal.states._stage_state import StageState  # noqa
 from snapcraft.internal.states._build_state import BuildState  # noqa
-from snapcraft.internal.states._pull_state import PullState  # noqa
+from snapcraft.internal.states._pull_state import PullState    # noqa
