@@ -18,8 +18,9 @@ import yaml
 
 
 class State(yaml.YAMLObject):
-    def __init__(self, options):
+    def __init__(self, options, project):
         self.properties = self.properties_of_interest(options)
+        self.project_options = self.project_options_of_interest(project)
 
     def properties_of_interest(self, options):
         """Extract the properties concerning this step from the options.
@@ -29,6 +30,11 @@ class State(yaml.YAMLObject):
 
         raise NotImplementedError
 
+    def project_options_of_interest(self, project):
+        """Extract the options concerning this step from the project."""
+
+        raise NotImplementedError
+
     def __repr__(self):
         items = sorted(self.__dict__.items())
         strings = (': '.join((key, repr(value))) for key, value in items)
