@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015, 2016 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -14,9 +14,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+
+from snapcraft.internal.sources import Script      # noqa
 from snapcraft.internal.sources import Bazaar      # noqa
+from snapcraft.internal.sources import Deb         # noqa
 from snapcraft.internal.sources import Git         # noqa
 from snapcraft.internal.sources import Mercurial   # noqa
+from snapcraft.internal.sources import Subversion  # noqa
 from snapcraft.internal.sources import Tar         # noqa
 from snapcraft.internal.sources import Local       # noqa
+from snapcraft.internal.sources import Zip         # noqa
 from snapcraft.internal.sources import get         # noqa
