@@ -41,6 +41,8 @@ class ExecutionTestCases(tests.TestCase):
 version: 0
 summary: test
 description: test
+confinement: strict
+grade: stable
 {type}
 
 {parts}
@@ -48,6 +50,25 @@ description: test
 
         super().make_snapcraft_yaml(yaml.format(parts=parts, type=snap_type))
 
+    def test__replace_in_parts(self):
+        class Options:
+            def __init__(self):
+                self.source = '$SNAPCRAFT_PART_INSTALL'
+
+        class Code:
+            def __init__(self):
+                self.options = Options()
+                self.installdir = '/tmp'
+
+        class Part:
+            def __init__(self):
+                self.code = Code()
+
+        part = Part()
+        new_part = lifecycle._replace_in_part(part)
+
+        self.assertEqual(part.code.installdir, new_part.code.options.source)
+
     def test_exception_when_dependency_is_required(self):
         self.make_snapcraft_yaml("""parts:
   part1:
@@ -87,6 +108,10 @@ description: test
                               part_names=['part2'])
 
         self.assertEqual(
+            'Skipping pull part1 (already ran)\n'
+            'Skipping build part1 (already ran)\n'
+            'Skipping stage part1 (already ran)\n'
+            'Skipping prime part1 (already ran)\n'
             'Preparing to pull part2 \n'
             'Pulling part2 \n',
             self.fake_logger.output)
@@ -99,6 +124,10 @@ description: test
     plugin: nil
     after:
       - part1
+  part3:
+    plugin: nil
+    after:
+      - part2
 """)
 
         snap_info = lifecycle.execute('pull', self.project_options)
@@ -115,12 +144,17 @@ description: test
             'Preparing to pull part1 \n'
             'Pulling part1 \n'
             '\'part2\' has prerequisites that need to be staged: part1\n'
-            'Skipping pull part1 (already ran)\n'
             'Preparing to build part1 \n'
             'Building part1 \n'
             'Staging part1 \n'
             'Preparing to pull part2 \n'
-            'Pulling part2 \n',
+            'Pulling part2 \n'
+            '\'part3\' has prerequisites that need to be staged: part2\n'
+            'Preparing to build part2 \n'
+            'Building part2 \n'
+            'Staging part2 \n'
+            'Preparing to pull part3 \n'
+            'Pulling part3 \n',
             self.fake_logger.output)
 
     def test_os_type_returned_by_lifecycle(self):
@@ -143,7 +177,7 @@ description: test
         }
         self.assertEqual(snap_info, expected_snap_info)
 
-    def test_dirty_strip_restrips_single_part(self):
+    def test_dirty_prime_reprimes_single_part(self):
         self.make_snapcraft_yaml("""parts:
   part1:
     plugin: nil
@@ -152,20 +186,20 @@ description: test
 """)
 
         # Strip it.
-        lifecycle.execute('strip', self.project_options)
+        lifecycle.execute('prime', self.project_options)
 
         # Reset logging since we only care about the following
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
         def _fake_is_dirty(self, step):
-            return self.name == 'part1' and step == 'strip'
+            return self.name == 'part1' and step == 'prime'
 
-        # Should automatically clean and re-strip if that step is dirty
+        # Should automatically clean and re-prime if that step is dirty
         # for the part.
         with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
                                _fake_is_dirty):
-            lifecycle.execute('strip', self.project_options)
+            lifecycle.execute('prime', self.project_options)
 
         output = self.fake_logger.output.split('\n')
         part1_output = [line.strip() for line in output if 'part1' in line]
@@ -176,7 +210,7 @@ description: test
                 'Skipping pull part2 (already ran)',
                 'Skipping build part2 (already ran)',
                 'Skipping stage part2 (already ran)',
-                'Skipping strip part2 (already ran)',
+                'Skipping prime part2 (already ran)',
             ],
             part2_output)
 
@@ -185,12 +219,12 @@ description: test
                 'Skipping pull part1 (already ran)',
                 'Skipping build part1 (already ran)',
                 'Skipping stage part1 (already ran)',
-                'Cleaning snapping area for part1 (out of date)',
-                'Stripping part1',
+                'Cleaning priming area for part1 (out of date)',
+                'Priming part1',
             ],
             part1_output)
 
-    def test_dirty_strip_restrips_multiple_part(self):
+    def test_dirty_prime_reprimes_multiple_part(self):
         self.make_snapcraft_yaml("""parts:
   part1:
     plugin: nil
@@ -199,20 +233,20 @@ description: test
 """)
 
         # Strip it.
-        lifecycle.execute('strip', self.project_options)
+        lifecycle.execute('prime', self.project_options)
 
         # Reset logging since we only care about the following
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
         def _fake_is_dirty(self, step):
-            return step == 'strip'
+            return step == 'prime'
 
-        # Should automatically clean and re-strip if that step is dirty
+        # Should automatically clean and re-prime if that step is dirty
         # for the part.
         with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
                                _fake_is_dirty):
-            lifecycle.execute('strip', self.project_options)
+            lifecycle.execute('prime', self.project_options)
 
         output = self.fake_logger.output.split('\n')
         part1_output = [line.strip() for line in output if 'part1' in line]
@@ -223,8 +257,8 @@ description: test
                 'Skipping pull part2 (already ran)',
                 'Skipping build part2 (already ran)',
                 'Skipping stage part2 (already ran)',
-                'Cleaning snapping area for part2 (out of date)',
-                'Stripping part2',
+                'Cleaning priming area for part2 (out of date)',
+                'Priming part2',
             ],
             part2_output)
 
@@ -233,8 +267,8 @@ description: test
                 'Skipping pull part1 (already ran)',
                 'Skipping build part1 (already ran)',
                 'Skipping stage part1 (already ran)',
-                'Cleaning snapping area for part1 (out of date)',
-                'Stripping part1',
+                'Cleaning priming area for part1 (out of date)',
+                'Priming part1',
             ],
             part1_output)
 
@@ -278,7 +312,7 @@ description: test
             [
                 'Skipping pull part1 (already ran)',
                 'Skipping build part1 (already ran)',
-                'Skipping cleaning snapping area for part1 (out of date) '
+                'Skipping cleaning priming area for part1 (out of date) '
                 '(already clean)',
                 'Cleaning staging area for part1 (out of date)',
                 'Staging part1',
@@ -317,7 +351,7 @@ description: test
             [
                 'Skipping pull part2 (already ran)',
                 'Skipping build part2 (already ran)',
-                'Skipping cleaning snapping area for part2 (out of date) '
+                'Skipping cleaning priming area for part2 (out of date) '
                 '(already clean)',
                 'Cleaning staging area for part2 (out of date)',
                 'Staging part2',
@@ -328,7 +362,7 @@ description: test
             [
                 'Skipping pull part1 (already ran)',
                 'Skipping build part1 (already ran)',
-                'Skipping cleaning snapping area for part1 (out of date) '
+                'Skipping cleaning priming area for part1 (out of date) '
                 '(already clean)',
                 'Cleaning staging area for part1 (out of date)',
                 'Staging part1',
@@ -407,20 +441,24 @@ description: test
         self.assertEqual(
             'Skipping pull part1 (already ran)\n'
             'Skipping build part1 (already ran)\n'
-            'Skipping cleaning snapping area for part1 (out of date) '
+            'Skipping cleaning priming area for part1 (out of date) '
             '(already clean)\n'
             'Cleaning staging area for part1 (out of date)\n'
+            'Skipping cleaning priming area for part2 (out of date) '
+            '(already clean)\n'
+            'Skipping cleaning staging area for part2 (out of date) '
+            '(already clean)\n'
             'Staging part1 \n',
             self.fake_logger.output)
 
-    def test_dirty_stage_restrips(self):
+    def test_dirty_stage_reprimes(self):
         self.make_snapcraft_yaml("""parts:
   part1:
     plugin: nil
 """)
 
         # Strip it.
-        lifecycle.execute('strip', self.project_options)
+        lifecycle.execute('prime', self.project_options)
 
         # Reset logging since we only care about the following
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
@@ -433,15 +471,15 @@ description: test
         # for the part.
         with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
                                _fake_is_dirty):
-            lifecycle.execute('strip', self.project_options)
+            lifecycle.execute('prime', self.project_options)
 
         self.assertEqual(
             'Skipping pull part1 (already ran)\n'
             'Skipping build part1 (already ran)\n'
-            'Cleaning snapping area for part1 (out of date)\n'
+            'Cleaning priming area for part1 (out of date)\n'
             'Cleaning staging area for part1 (out of date)\n'
             'Staging part1 \n'
-            'Stripping part1 \n',
+            'Priming part1 \n',
             self.fake_logger.output)
 
     def test_dirty_build_raises(self):
@@ -502,6 +540,37 @@ description: test
             "The 'pull' step of 'part1' is out of date. Please clean that "
             "part's 'pull' step in order to rebuild", str(raised.exception))
 
+    @mock.patch.object(snapcraft.BasePlugin, 'enable_cross_compilation')
+    @mock.patch('snapcraft.repo.install_build_packages')
+    def test_pull_is_dirty_if_target_arch_changes(
+            self, mock_install_build_packages, mock_enable_cross_compilation):
+        self.make_snapcraft_yaml("""parts:
+  part1:
+    plugin: nil
+""")
+
+        # Pull it with amd64
+        lifecycle.execute('pull', snapcraft.ProjectOptions(
+            target_deb_arch='amd64'))
+
+        # Reset logging since we only care about the following
+        self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
+        self.useFixture(self.fake_logger)
+
+        # Pull it again with armhf. Should catch that the part needs to be
+        # re-pulled due to the change in target architecture and raise an
+        # error.
+        with self.assertRaises(RuntimeError) as raised:
+            lifecycle.execute('pull', snapcraft.ProjectOptions(
+                target_deb_arch='armhf'))
+
+        self.assertEqual("Setting target machine to 'armhf'\n",
+                         self.fake_logger.output)
+
+        self.assertEqual(
+            "The 'pull' step of 'part1' is out of date. Please clean that "
+            "part's 'pull' step in order to rebuild", str(raised.exception))
+
 
 class HumanizeListTestCases(tests.TestCase):
 
