@@ -61,6 +61,54 @@ class CreateTest(tests.TestCase):
 
         self.assertEqual(y, expected)
 
+    def test_create_meta_with_confinement(self):
+        confinement_types = [
+            'strict',
+            'devmode',
+        ]
+
+        for confinement_type in confinement_types:
+            with self.subTest(key=confinement_type):
+                self.config_data['confinement'] = confinement_type
+
+                create_snap_packaging(
+                    self.config_data, self.snap_dir, self.parts_dir)
+
+                self.assertTrue(
+                    os.path.exists(self.snap_yaml),
+                    'snap.yaml was not created')
+
+                with open(self.snap_yaml) as f:
+                    y = yaml.load(f)
+                self.assertTrue(
+                    'confinement' in y,
+                    'Expected "confinement" property to be in snap.yaml')
+                self.assertEqual(y['confinement'], confinement_type)
+
+    def test_create_meta_with_grade(self):
+        grade_types = [
+            'stable',
+            'devel',
+        ]
+
+        for grade_type in grade_types:
+            with self.subTest(key=grade_type):
+                self.config_data['grade'] = grade_type
+
+                create_snap_packaging(
+                    self.config_data, self.snap_dir, self.parts_dir)
+
+                self.assertTrue(
+                    os.path.exists(self.snap_yaml),
+                    'snap.yaml was not created')
+
+                with open(self.snap_yaml) as f:
+                    y = yaml.load(f)
+                self.assertTrue(
+                    'grade' in y,
+                    'Expected "grade" property to be in snap.yaml')
+                self.assertEqual(y['grade'], grade_type)
+
     def test_create_meta_with_declared_license(self):
         open(os.path.join(os.curdir, 'LICENSE'), 'w').close()
         self.config_data['license'] = 'LICENSE'
@@ -79,6 +127,36 @@ class CreateTest(tests.TestCase):
         self.assertFalse('license' in y,
                          'license found in snap.yaml {}'.format(y))
 
+    def test_create_meta_with_epoch(self):
+        self.config_data['epoch'] = '1*'
+
+        create_snap_packaging(self.config_data, self.snap_dir, self.parts_dir)
+
+        self.assertTrue(
+            os.path.exists(self.snap_yaml), 'snap.yaml was not created')
+
+        with open(self.snap_yaml) as f:
+            y = yaml.load(f)
+        self.assertTrue(
+            'epoch' in y,
+            'Expected "epoch" property to be copied into snap.yaml')
+        self.assertEqual(y['epoch'], '1*')
+
+    def test_create_meta_with_assumes(self):
+        self.config_data['assumes'] = ['feature1', 'feature2']
+
+        create_snap_packaging(self.config_data, self.snap_dir, self.parts_dir)
+
+        self.assertTrue(
+            os.path.exists(self.snap_yaml), 'snap.yaml was not created')
+
+        with open(self.snap_yaml) as f:
+            y = yaml.load(f)
+        self.assertTrue(
+            'assumes' in y,
+            'Expected "assumes" property to be copied into snap.yaml')
+        self.assertEqual(y['assumes'], ['feature1', 'feature2'])
+
     def test_create_meta_with_declared_license_and_setup(self):
         open(os.path.join(os.curdir, 'LICENSE'), 'w').close()
         self.config_data['license'] = 'LICENSE'
@@ -288,7 +366,7 @@ PATH={0}/part1/install/usr/bin:{0}/part1/install/bin
 
         # Check that the wrapper is created even if there is already a file
         # with the same name.
-        open(os.path.join('snap', 'test_relexepath.wrapper'), 'w').close()
+        open(os.path.join('prime', 'test_relexepath.wrapper'), 'w').close()
 
         relative_wrapper_path = self.packager._wrap_exe(relative_exe_path)
         wrapper_path = os.path.join(self.snap_dir, relative_wrapper_path)
@@ -444,8 +522,8 @@ PATH={0}/part1/install/usr/bin:{0}/part1/install/bin
         with self.assertRaises(EnvironmentError) as raised:
             self.packager._wrap_apps(apps)
         self.assertEqual(
-            "The specified command 'command-does-not-exist' defined in 'app1' "
-            "does not exist or is not executable",
+            "The specified command 'command-does-not-exist' defined in the "
+            "app 'app1' does not exist or is not executable",
             str(raised.exception))
 
     def test_command_is_not_executable(self):
@@ -460,8 +538,8 @@ PATH={0}/part1/install/usr/bin:{0}/part1/install/bin
         with self.assertRaises(EnvironmentError) as raised:
             self.packager._wrap_apps(apps)
         self.assertEqual(
-            "The specified command 'command-not-executable' defined in 'app1' "
-            "does not exist or is not executable",
+            "The specified command 'command-not-executable' defined in the "
+            "app 'app1' does not exist or is not executable",
             str(raised.exception))
 
     def test_command_found(self):
